from osmdiff import Way
from osmdiff.osm import OSMObject
from typing_extensions import assert_type


class TestWay:
    "tests for Way object"

    def test_init_way(self):
        "Test Way init"
        way = Way()
        assert_type(way, Way)
        assert_type(way, OSMObject)
        assert_type(way.attribs, dict)
        assert_type(way.tags, dict)
        assert len(way.tags) == 0
        assert len(way.attribs) == 0
        assert_type(way.nodes, list)
        assert len(way.nodes) == 0



from osmdiff import AugmentedDiff, OSMChange
from typing_extensions import assert_type


class TestApi:
    "tests for API calls"

    state = None

    def test_osm_diff_api(self):
        "Test getting state from OSM API"
        osm_change = OSMChange()
        assert osm_change.get_state()
        self.state = osm_change.sequence_number
        assert_type(self.state, int)
        status = osm_change.retrieve()
        assert status == 200

    def test_augmented_diff_api(self):
        "Test getting augmented diff from Overpass API"
        augmented_diff = AugmentedDiff()
        assert augmented_diff.get_state()
        self.state = augmented_diff.sequence_number
        assert_type(self.state, int)
        status = augmented_diff.retrieve()
        assert status == 200



import os
from gzip import GzipFile
from xml.etree import ElementTree

import requests

from osmdiff.osm import OSMObject

REPLICATION_URL = "https://planet.openstreetmap.org/replication"


class OSMChange(object):
    def __init__(
        self,
        url=REPLICATION_URL,
        frequency="minute",
        file=None,
        sequence_number=None,
        debug=False,
    ):
        self.base_url = url
        self.debug = debug
        self.create = []
        self.modify = []
        self.delete = []
        if file:
            with open(file, "r") as fh:
                self._parse_stream(fh)
        else:
            self._frequency = frequency
            self._sequence_number = sequence_number

    def get_state(self):
        """Get the current state from the OSM API"""
        state_url = os.path.join(self.base_url, self._frequency, "state.txt")
        response = requests.get(state_url, timeout=5)
        if response.status_code != 200:
            return False
        for line in response.text.split("\n"):
            if self.debug:
                print(line)
            if line.startswith("sequenceNumber"):
                self._sequence_number = int(line[15:])
        return True

    def _build_sequence_url(self):
        seqno = str(self._sequence_number).zfill(9)
        url = os.path.join(
            self.base_url,
            self._frequency,
            seqno[:3],
            seqno[3:6],
            "{}{}".format(seqno[6:], ".osc.gz"),
        )
        if self.debug:
            print(url)
        return url

    def _parse_stream(self, stream):
        for event, elem in ElementTree.iterparse(stream):
            # if self.debug:
            #     print(event, elem)
            if elem.tag in ("create", "modify", "delete"):
                self._build_action(elem)
                if self.debug:
                    print("======={action}========".format(action=elem.tag))

    def _build_action(self, elem):
        for thing in elem:
            o = OSMObject.from_xml(thing)
            self.__getattribute__(elem.tag).append(o)
            if self.debug:
                print(o)
                print(o.attribs)
                print(o.tags)
                print(o.bounds)

    def retrieve(self, clear_cache=False) -> int:
        """
        Retrieve the OSM diff corresponding to the OSMChange sequence_number.
        """
        if not self._sequence_number:
            raise Exception("invalid sequence number")
        if clear_cache:
            self.create, self.modify, self.delete = ([], [], [])
        try:
            r = requests.get(self._build_sequence_url(), stream=True, timeout=30)
            if r.status_code != 200:
                return r.status_code
            gzfile = GzipFile(fileobj=r.raw)
            self._parse_stream(gzfile)
            return r.status_code
        except ConnectionError:
            # FIXME catch this?
            return 0

    @classmethod
    def from_xml(cls, path):
        new_osmchange_obj = cls()
        new_osmchange_obj._parse_stream(path)
        return new_osmchange_obj

    def sequence_number(self):
        return self._sequence_number

    def set_sequence_number(self, sn):
        self._sequence_number = int(sn)

    sequence_number = property(sequence_number, set_sequence_number)

    def __repr__(self):
        return "OSMChange ({create} created, {modify} modified, \
{delete} deleted)".format(
            create=len(self.create), modify=len(self.modify), delete=len(self.delete)
        )



#!/usr/bin/env python

from osmdiff import AugmentedDiff, OSMChange
from osmdiff.osm import Node, Relation, Way

debug = True

r = OSMChange(debug=debug)
r.get_state()
r.retrieve()
print(r)

r = OSMChange(file="test_osmchange.xml", debug=debug)
print(r)

a = AugmentedDiff(file="test_adiff.xml", debug=debug)
print(a)

a = AugmentedDiff(
    # minlon=-160.0,
    # minlat=20.0,
    # maxlon=-80.0,
    # maxlat=60.0,
    debug=debug
)
a.get_state()
a.retrieve()
print(a)

# n = Node()
# w = Way()
# r = Relation()

# r = replication.OSMChange(frequency="hour")

# print(r.sequence_number)
# r.get(r.sequence_number)

# r = replication.OSMChange(frequency="day")

# print(r.sequence_number)
# r.get(r.sequence_number)



from pathlib import Path

from osmdiff import Node, OSMChange, Relation, Way
from typing_extensions import assert_type


class TestOSMChange:
    "tests for OSMChange object"

    osmchange_file_path = (
        Path(__file__).parent / "fixtures/test_osmchange.xml"
    ).resolve()

    def test_init_osmchange(self):
        "Test OSMChange init"
        osmchange = OSMChange()
        assert_type(osmchange, OSMChange)
        assert_type(osmchange.create, list)
        assert_type(osmchange.modify, list)
        assert_type(osmchange.delete, list)
        assert len(osmchange.create) == 0
        assert len(osmchange.modify) == 0
        assert len(osmchange.delete) == 0

    def test_set_sequencenumber(self):
        "Sequence number is not defined by default but can be set manually"
        osm_change = OSMChange()
        assert not osm_change.sequence_number
        osm_change.sequence_number = 12345
        assert osm_change.sequence_number == 12345
        osm_change.sequence_number = "12345"
        assert osm_change.sequence_number == 12345

    def test_3_readfromfile(self):
        "Test initializing from file"
        osmchange = OSMChange.from_xml(self.osmchange_file_path)
        assert len(osmchange.create) == 831
        assert len(osmchange.modify) == 368
        assert len(osmchange.delete) == 3552
        nodes_created = [o for o in osmchange.create if isinstance(o, Node)]
        ways_created = [o for o in osmchange.create if isinstance(o, Way)]
        rels_created = [o for o in osmchange.create if isinstance(o, Relation)]
        assert len(nodes_created) == 699
        assert len(ways_created) == 132
        assert len(rels_created) == 0
        assert len(nodes_created + ways_created + rels_created) == len(osmchange.create)
